@@ -2,7 +2,7 @@
    Project: LaternaMagica
    AppController.m
 
-   Copyright (C) 2006-2007 Riccardo Mottola
+   Copyright (C) 2006-2009 Riccardo Mottola
 
    Author: Riccardo Mottola
 
@@ -48,7 +48,7 @@
     view = smallView;
 
     frame = [[NSScreen mainScreen] frame];
-    fullWindow = [[NSWindow alloc] initWithContentRect: frame
+    fullWindow = [[LMWindow alloc] initWithContentRect: frame
                                              styleMask: NSBorderlessWindowMask
                                                backing: NSBackingStoreBuffered
                                                  defer: NO];
@@ -63,10 +63,12 @@
     [fullView setImageScaling: NSScaleNone];
     [fullView setImageAlignment:NSImageAlignCenter];
     [fullView setAutoresizingMask: NSViewWidthSizable | NSViewHeightSizable];
+    [fullView setController:self];
     
 
     // avoid replacing the contentview with a NSControl subclass, thus add a subview instead
     [[fullWindow contentView] addSubview: fullView];
+    [fullWindow setInitialFirstResponder:fullView];
 }
 
 - (void)dealloc
@@ -102,9 +104,11 @@
     files = [openPanel filenames];
     e = [files objectEnumerator];
     fmgr = [NSFileManager defaultManager];
-    while ((filename = (NSString*)[e nextObject])) {
+    while ((filename = (NSString*)[e nextObject]))
+    {
         attrs = [fmgr fileAttributesAtPath:filename traverseLink:YES];
         if (attrs)
+        {
             if ([attrs objectForKey:NSFileType] == NSFileTypeDirectory)
             {
                 NSArray      *dirContents;
@@ -117,22 +121,27 @@
                 while ((filename2 = (NSString*)[e2 nextObject]))
                 {
                     NSString *tempName;
+                    NSString *lastPathComponent;
 
+                    lastPathComponent = [filename2 lastPathComponent];
                     tempName = [filename stringByAppendingPathComponent:filename2];
                     attrs2 = [[NSFileManager defaultManager] fileAttributesAtPath:tempName traverseLink:YES];
                     if (attrs2)
                     {
                         if ([attrs2 objectForKey:NSFileType] != NSFileTypeDirectory)
                         {
-                            [self addFile:tempName];
+                            if (!([lastPathComponent isEqualToString:@".gwdir"] || [lastPathComponent isEqualToString:@".DS_Store"]))
+                            {
+                                [self addFile:tempName];
+                            }
                         }
                     }
                 }
             } else
-            {
+            { /* not a directory */
                 [self addFile:filename];
             }
-        else
+        } else
         {
             NSLog(@"open panel did not return a valid path");
         }
@@ -206,7 +215,7 @@
     [self scaleView:[view image]];
 }
 
-// method called as a notification from the selection change
+/** method called as a notification from the selection change */
 - (void)_selectionDidChange :(NSNotification *)notif
 {
     NSTableView *table;
@@ -218,21 +227,25 @@
         [self changeImage:[fileListData pathAtIndex:selectedRow]];
 }
 
-// method called as a notification from the window resize
-// or if scale preferences changed
+/** method called as a notification from the window resize
+  or if scale preferences changed */
 - (void)_windowDidResize :(NSNotification *)notif
 {
     [self scaleView: [view image]];
 }
 
 
-- (IBAction)setFullScreen :(id)sender
+- (IBAction)setFullScreen: (id)sender
 {
     NSImage *image;
 
     image = [view image];
+    
+    /* we choose not to respond to key events if not in fullscreen */
+    if ([sender isKindOfClass:[NSEvent class]] && [fullScreenMenuItem state] == NSOffState)
+        return;
 
-    // check the sender and set the other item accordingly
+    /* check the sender and set the other item accordingly */    
     if (sender == fullScreenButton)
         [fullScreenMenuItem setState:[fullScreenButton state]];
     else
@@ -257,7 +270,7 @@
     }
     [self setScaleToFit: self];
     [self scaleView: image];
-    [view  setImage: image];
+    [view setImage: image];
     [view setNeedsDisplay:YES];
     [[view superview] setNeedsDisplay:YES];
     [window displayIfNeeded];
@@ -290,7 +303,8 @@
 
     rows = [fileListView numberOfRows];
 
-    if (rows > 0) {
+    if (rows > 0)
+    {
         sr = [fileListView selectedRow];
 
         if (sr < (rows - 1))
@@ -311,24 +325,28 @@
     if (rows >= 0)
     {
         sr = [fileListView selectedRow];
-        [fileListData removeObjectAtIndex: sr];
-        [fileListView reloadData];
+        if (sr >= 0)
+        {
+            [fileListData removeObjectAtIndex: sr];
+            [fileListView reloadData];
 
-        rows = [fileListView numberOfRows];
-        if (rows > 0)
-        {
-            // if we remove the last image, the selection changes
-            // otherwise no selection change notification is generated
-            // and thus we update simply the image
-            if (sr >= rows)
-                [fileListView selectRow: rows-1 byExtendingSelection: NO];
-            else
-                [self changeImage: [fileListData pathAtIndex: sr]];
+            rows = [fileListView numberOfRows];
+            if (rows > 0)
+            {
+                // if we remove the last image, the selection changes
+                // otherwise no selection change notification is generated
+                // and thus we update simply the image
+                if (sr >= rows)
+                    [fileListView selectRow: rows-1 byExtendingSelection: NO];
+                else
+                    [self changeImage: [fileListData pathAtIndex: sr]];
             
-        } else
-        {
-            // no image to select, we clear the display
-            [view setImage: nil];
+            } else
+            {
+                // no image to select, we clear the display
+                [view setImage: nil];
+                [window setTitle:@"None"]; 
+            }
         }
     }
 }
@@ -344,7 +362,7 @@
             NSFileManager *fm;
             
             fm = [NSFileManager defaultManager];
-            // TODO should implement a handelr and error messages
+            // TODO should implement a handler and error messages
             [fm removeFileAtPath:[fileListData pathAtIndex:sr] handler:nil];
             
             [self removeImage:self];
@@ -360,7 +378,8 @@
     int x, y;
     int w, h;
     int s;
-    int bytesPerPixel;
+    int srcSamplesPerPixel;
+    int destSamplesPerPixel;
     unsigned char *srcData;
     unsigned char *destData;
     unsigned char *p1, *p2;
@@ -371,8 +390,8 @@
 
     w = [srcImageRep pixelsWide];
     h = [srcImageRep pixelsHigh];
-    bytesPerPixel = [srcImageRep bitsPerPixel] /8;
-
+    srcSamplesPerPixel = [srcImageRep samplesPerPixel];
+    destSamplesPerPixel = srcSamplesPerPixel;
     destImage = [[NSImage alloc] initWithSize:NSMakeSize(h, w)]; /* we swap h and w */
     destImageRep = [[NSBitmapImageRep alloc]
                 initWithBitmapDataPlanes:NULL
@@ -383,7 +402,7 @@
                                 hasAlpha:[srcImageRep hasAlpha]
                                 isPlanar:[srcImageRep isPlanar]
                           colorSpaceName:[srcImageRep colorSpaceName]
-                             bytesPerRow:0
+                             bytesPerRow:h*destSamplesPerPixel  // we need to set this because otherwise mac > 10.4 will set a padded value
                             bitsPerPixel:0];
 
     srcData = [srcImageRep bitmapData];
@@ -392,9 +411,9 @@
     for (y = 0; y < h; y++)
         for (x = 0; x < w; x++)
         {
-            p1 = srcData + bytesPerPixel * (y * w + x);
-            p2 = destData + bytesPerPixel * ((w-x-1) * h + y);
-            for (s = 0; s < bytesPerPixel; s++)
+            p1 = srcData + srcSamplesPerPixel * (y * w + x);
+            p2 = destData + destSamplesPerPixel * ((w-x-1) * h + y);
+            for (s = 0; s < srcSamplesPerPixel; s++)
                 p2[s] = p1[s];
         }
 
@@ -418,7 +437,8 @@
     int x, y;
     int w, h;
     int s;
-    int bytesPerPixel;
+    int srcSamplesPerPixel;
+    int destSamplesPerPixel;
     unsigned char *srcData;
     unsigned char *destData;
     unsigned char *p1, *p2;
@@ -429,7 +449,8 @@
 
     w = [srcImageRep pixelsWide];
     h = [srcImageRep pixelsHigh];
-    bytesPerPixel = [srcImageRep bitsPerPixel] /8;
+    srcSamplesPerPixel = [srcImageRep samplesPerPixel];
+    destSamplesPerPixel = srcSamplesPerPixel;
 
     destImage = [[NSImage alloc] initWithSize:NSMakeSize(w, h)];
     destImageRep = [[NSBitmapImageRep alloc]
@@ -441,7 +462,7 @@
                                 hasAlpha:[srcImageRep hasAlpha]
                                 isPlanar:[srcImageRep isPlanar]
                           colorSpaceName:[srcImageRep colorSpaceName]
-                             bytesPerRow:0
+                             bytesPerRow:w*destSamplesPerPixel
                             bitsPerPixel:0];
 
     srcData = [srcImageRep bitmapData];
@@ -450,9 +471,9 @@
     for (y = 0; y < h; y++)
         for (x = 0; x < w; x++)
         {
-            p1 = srcData + bytesPerPixel * (y * w + x);
-            p2 = destData + bytesPerPixel * ((h-y-1) * w + (w-x-1));
-            for (s = 0; s < bytesPerPixel; s++)
+            p1 = srcData + srcSamplesPerPixel * (y * w + x);
+            p2 = destData + srcSamplesPerPixel * ((h-y-1) * w + (w-x-1));
+            for (s = 0; s < srcSamplesPerPixel; s++)
                 p2[s] = p1[s];
         }
 
@@ -475,7 +496,8 @@
     int x, y;
     int w, h;
     int s;
-    int bytesPerPixel;
+    int srcSamplesPerPixel;
+    int destSamplesPerPixel;
     unsigned char *srcData;
     unsigned char *destData;
     unsigned char *p1, *p2;
@@ -486,7 +508,8 @@
 
     w = [srcImageRep pixelsWide];
     h = [srcImageRep pixelsHigh];
-    bytesPerPixel = [srcImageRep bitsPerPixel] /8;
+    srcSamplesPerPixel = [srcImageRep samplesPerPixel];
+    destSamplesPerPixel = srcSamplesPerPixel;
 
     destImage = [[NSImage alloc] initWithSize:NSMakeSize(h, w)]; /* we swap h and w */
     destImageRep = [[NSBitmapImageRep alloc]
@@ -498,7 +521,7 @@
                                 hasAlpha:[srcImageRep hasAlpha]
                                 isPlanar:[srcImageRep isPlanar]
                           colorSpaceName:[srcImageRep colorSpaceName]
-                             bytesPerRow:0
+                             bytesPerRow:h*destSamplesPerPixel
                             bitsPerPixel:0];
 
     srcData = [srcImageRep bitmapData];
@@ -507,9 +530,9 @@
     for (y = 0; y < h; y++)
         for (x = 0; x < w; x++)
         {
-            p1 = srcData + bytesPerPixel * (y * w + x);
-            p2 = destData + bytesPerPixel * (x * h + (h-y-1));
-            for (s = 0; s < bytesPerPixel; s++)
+            p1 = srcData + srcSamplesPerPixel * (y * w + x);
+            p2 = destData + destSamplesPerPixel * (x * h + (h-y-1));
+            for (s = 0; s < srcSamplesPerPixel; s++)
                 p2[s] = p1[s];
         }
 
@@ -524,4 +547,113 @@
     [destImage release];
 }
 
+- (BOOL)validateMenuItem:(id)sender
+{
+    /* the menu item returned is not the same object, so we use isEqual */
+    if ([sender isEqual:saveAsMenuItem])
+    {
+        if ([view image] == nil)
+            return NO;
+    }
+    return YES;
+}
+
+- (IBAction)saveImageAs:(id)sender
+{
+    NSImage *srcImage;
+    NSBitmapImageRep *srcImageRep;
+    NSData *dataOfRep;
+    NSDictionary *repProperties;
+    NSString *origFileName;
+    NSString *filenameNoExtension;
+    
+
+    origFileName = [[fileListData pathAtIndex:[fileListView selectedRow]] lastPathComponent];
+    filenameNoExtension = [origFileName stringByDeletingPathExtension];
+    savePanel = [NSSavePanel savePanel];
+    [savePanel setDelegate: self];
+    /* if the accessory view comes from a window it needs a retain */
+    [savePanel setAccessoryView: [saveOptionsView retain]];
+
+    /* simulate clicks to be sure interface is consistent */
+    [jpegCompressionSlider performClick:nil];
+    [fileTypePopUp sendAction:@selector(setCompressionType:) to:self];
+
+    if ([savePanel runModalForDirectory:@"" file:filenameNoExtension] ==  NSFileHandlingPanelOKButton)
+    {
+        NSString *fileName;
+        int selItem;
+
+        srcImage = [view image];
+        srcImageRep = [NSBitmapImageRep imageRepWithData:[srcImage TIFFRepresentation]];
+
+
+        fileName = [savePanel filename];
+                
+        selItem = [fileTypePopUp indexOfSelectedItem];
+        if (selItem == 0)
+        {
+            NSLog(@"Tiff");
+            repProperties = [NSDictionary dictionaryWithObject:[NSNumber numberWithInt:NSTIFFCompressionLZW] forKey:NSImageCompressionMethod];
+            dataOfRep = [srcImageRep representationUsingType: NSTIFFFileType properties:repProperties];
+
+        } else if (selItem == 1)
+        {
+            NSLog(@"Jpeg");
+            repProperties = [NSDictionary dictionaryWithObject:[NSNumber numberWithFloat:[jpegCompressionSlider floatValue]/100] forKey:NSImageCompressionFactor];
+            dataOfRep = [srcImageRep representationUsingType: NSJPEGFileType properties:repProperties];
+        }
+        [dataOfRep writeToFile:fileName atomically:NO];            
+    }
+//    [savePanel release];
+}
+
+
+/* ===== delegates =====*/
+- (BOOL)application:(NSApplication *)theApplication openFile:(NSString *)filename
+{
+    [self addFile:filename];
+    return YES;
+}
+
+/* save panel delegates */
+/** change the file type */
+- (IBAction)setCompressionType:(id)sender
+{
+    int selItem;
+    
+    selItem = [fileTypePopUp indexOfSelectedItem];
+    if (selItem == 0)
+    {
+        NSLog(@"Tiff");
+        [jpegCompressionSlider setEnabled:NO];
+        [jpegCompressionField setEnabled:NO];
+#if (MAC_OS_X_VERSION_MAX_ALLOWED <= MAC_OS_X_VERSION_10_2)
+        [savePanel setRequiredFileType:@"tiff"];
+#else
+        [savePanel setAllowedFileTypes:[NSArray arrayWithObjects: @"tif", @"tiff", nil]];
+#endif 
+    } else if (selItem == 1)
+    {
+        NSLog(@"Jpeg");
+        [jpegCompressionSlider setEnabled:YES];
+        [jpegCompressionField setEnabled:YES];
+#if (MAC_OS_X_VERSION_MAX_ALLOWED <= MAC_OS_X_VERSION_10_2)
+        [savePanel setRequiredFileType:@"jpg"];
+#else
+        [savePanel setAllowedFileTypes:[NSArray arrayWithObjects: @"jpg", @"jpeg", nil]];
+#endif 
+    }
+}
+
+/** keep the slider and the text view of the compression level in sync */
+- (IBAction)setCompressionLevel:(id)sender
+{
+    if (sender == jpegCompressionField)
+        [jpegCompressionSlider takeFloatValueFrom:sender];
+    else
+        [jpegCompressionField takeFloatValueFrom:sender];
+}
+
+
 @end
