@@ -2,7 +2,7 @@
    Project: LaternaMagica
    LMFlipView.m
 
-   Copyright (C) 2006-2007 Riccardo Mottola
+   Copyright (C) 2006-2009 Riccardo Mottola
 
    Author: Riccardo Mottola
 
@@ -24,7 +24,13 @@
 */
 
 #import "LMFlipView.h"
+#import "AppController.h"
 
+#ifdef GNUSTEP
+#define KC_ESCAPE 9
+#else
+#define KC_ESCAPE 53
+#endif
 
 @implementation LMFlipView
 
@@ -33,4 +39,28 @@
     return YES;
 }
 
+- (void)setController:(AppController *)aController
+{
+    controller = aController;
+}
+
+/* ---- Key event methods ---- */
+
+/** respond to key equivalents which are not bound do menu items */
+-(BOOL)performKeyEquivalent: (NSEvent*)theEvent
+{
+    unsigned short keyCode;
+
+    keyCode = [theEvent keyCode];
+    NSLog(@"keyCode %d", keyCode);
+    if (keyCode == KC_ESCAPE)
+    {
+        NSLog(@"(keyCode) Escape!");
+        [controller setFullScreen:theEvent];
+        return YES;
+    }
+    
+    return [super performKeyEquivalent:theEvent];
+}
+
 @end
